import operator
from pathlib import Path
from typing import List, Type
from prettytable import PrettyTable
from superglue.environment.config import JOBS_PATH, MODULES_PATH, NOTEBOOKS_PATH, TOOLS_PATH
from superglue.core.components.component_list import SuperglueComponentList
from superglue.core.components.makefile import SuperglueMakefile
from superglue.core.components.module import SuperglueModule
from superglue.core.components.job import SuperglueJob
from superglue.core.components.tests import SuperglueTests
from superglue.core.components.files import SuperglueFiles


class SuperglueProject:
    """Class represents the superglue project structure"""

    @property
    def jobs_path(self) -> Path:
        return JOBS_PATH

    @property
    def modules_path(self) -> Path:
        return MODULES_PATH

    @property
    def notebooks_path(self) -> Path:
        return NOTEBOOKS_PATH

    @property
    def tools_path(self) -> Path:
        return TOOLS_PATH

    @property
    def project_dirs(self) -> List[Path]:
        return [self.jobs_path, self.modules_path, self.notebooks_path, self.tools_path]

    @property
    def job(self) -> Type[SuperglueJob]:
        return SuperglueJob

    @property
    def jobs(self) -> SuperglueComponentList[SuperglueJob]:
        jobs = [self.job.get(p.name) for p in self.jobs_path.iterdir()]
        return SuperglueComponentList(jobs)

    @property
    def module(self) -> Type[SuperglueModule]:
        return SuperglueModule

    @property
    def modules(self) -> SuperglueComponentList[SuperglueModule]:
        modules = [self.module.get(p.name) for p in self.modules_path.iterdir()]
        return SuperglueComponentList(modules)

    @property
    def components(self) -> SuperglueComponentList:
        return SuperglueComponentList(self.jobs + self.modules)

    @property
    def makefile(self) -> Type[SuperglueMakefile]:
        return SuperglueMakefile

    @property
    def tests(self) -> Type[SuperglueTests]:
        return SuperglueTests

    @property
    def files(self) -> Type[SuperglueFiles]:
        return SuperglueFiles

    @property
    def pretty_table_fields(self) -> List[str]:
        return ["Component Name", "Component Type", "Local Stats", "s3 Status", "Version Status", "Version Number"]

    def is_locked(self) -> bool:
        return self.jobs.are_locked() and self.modules.are_locked()

    def versions_match(self) -> bool:
        return self.components.versions_match()

    def save_project_component(self, component_name: str) -> None:
        component_property = getattr(self, component_name)
        component = component_property.new()
        component.save()

    def save_base_project(self) -> None:
        for project_dir in self.project_dirs:
            project_dir.mkdir(exist_ok=True)

    def save_project_components(self) -> None:
        for component in "makefile", "files", "tests":
            self.save_project_component(component)

    def create(self) -> None:
        self.save_base_project()
        self.save_project_components()

    def get_pretty_table(self) -> PrettyTable:
        table = PrettyTable()
        table.field_names = self.pretty_table_fields
        table.sort_key = operator.itemgetter(0, 1)
        table.sortby = "Component Type"
        for field in self.pretty_table_fields:
            if field != "Version Number":
                table.align[field] = "l"
        return table



from typing import List, TypeVar
from pathlib import Path
from superglue.core.components.base import BaseSuperglueComponent


SuperglueFilesType = TypeVar("SuperglueFilesType", bound="SuperglueFiles")


class SuperglueFiles(BaseSuperglueComponent):
    def __init__(self) -> None:
        super(SuperglueFiles, self).__init__(
            root_dir=Path.cwd(), component_type="superglue_files", component_name="files"
        )

    @property
    def gitignore_file(self) -> Path:
        return self.root_dir / ".gitignore"

    @property
    def dotenv_file(self) -> Path:
        return self.root_dir / ".env"

    @property
    def files(self) -> List[Path]:
        return [self.gitignore_file, self.dotenv_file]

    @classmethod
    def new(cls) -> SuperglueFilesType:
        return cls()

    def create_files(self) -> None:
        for file in self.files:
            file.touch(exist_ok=True)

    def write_file_content(self) -> None:
        jinja = self.get_jinja_environment()
        for file in self.files:
            template = jinja.get_template(f"template{file.name}")
            file.write_text(template.render())

    def save(self) -> None:
        self.create_files()
        self.write_file_content()



import os
import yaml
import boto3
import botocore
from io import StringIO
from pathlib import Path
from typing import Dict, List, Optional, TypeVar, Any
from superglue.core.components.module import SuperglueModule
from superglue.core.components.base import SuperglueComponent
from superglue.environment.config import JOBS_PATH, MODULES_PATH
from superglue.core.components.component_list import SuperglueComponentList
from superglue.environment.variables import SUPERGLUE_S3_BUCKET, SUPERGLUE_IAM_ROLE
from superglue.core.components.tests import SuperglueTests
from copy import deepcopy

SuperglueJobType = TypeVar("SuperglueJobType", bound="SuperglueJob")


class NoAnchorsDumper(yaml.SafeDumper):
    def ignore_aliases(self, data):
        return True


class SuperglueJob(SuperglueComponent):
    def __init__(self, job_name: str, tests: Optional[SuperglueTests] = None, *args, **kwargs) -> None:
        self.tests = tests or SuperglueTests()

        super(SuperglueJob, self).__init__(
            *args,
            root_dir=JOBS_PATH,
            component_name=job_name,
            component_type="superglue_job",
            bucket=SUPERGLUE_S3_BUCKET,
            iam_role=SUPERGLUE_IAM_ROLE,
            **kwargs,
        )

        try:
            config_context = os.path.expandvars(self.config_file.read_text())
            self.config: Dict = yaml.safe_load(StringIO(config_context))
        except FileNotFoundError:
            self.config = {}

        self.deployment_config = {"job_configs": []}

    @property
    def job_path(self) -> Path:
        return self.root_dir / self.name

    @property
    def check_path(self) -> Path:
        return self.job_path / "check"

    @property
    def pys_path(self) -> Path:
        return self.job_path / "py"

    @property
    def jars_path(self) -> Path:
        return self.job_path / "jars"

    @property
    def shared_path(self) -> Path:
        return MODULES_PATH

    @property
    def job_test_path(self) -> Path:
        return self.tests.jobs_test_dir / self.name

    @property
    def job_tests_file(self) -> Path:
        return self.job_test_path / f"test_{self.name}.py"

    @property
    def main_script_file(self) -> Path:
        return self.job_path / "main.py"

    @property
    def config_file(self) -> Path:
        return self.job_path / "config_base.yml"

    @property
    def overrides_file(self) -> Path:
        return self.job_path / "config_overrides.yml"

    @property
    def deployment_config_file(self) -> Path:
        return self.job_path / "config_merged.yml"

    @property
    def superglue_modules(self) -> Dict[str, str]:
        return self.config.get("superglue_modules", {})

    @property
    def overrides(self) -> List[Dict]:
        try:
            return yaml.safe_load(self.overrides_file.open())["overrides"]
        except FileNotFoundError:
            pass
        except TypeError:
            pass
        return []

    @property
    def jar_files(self) -> List[Path]:
        try:
            return [p for p in self.jars_path.glob("**/*.jar")]
        except FileNotFoundError:
            return []

    @property
    def py_files(self) -> List[Path]:
        ext = ["*.py", "*.jar"]
        try:
            return [p for e in ext for p in self.pys_path.glob(f"**/{e}")]
        except FileNotFoundError:
            return []

    @property
    def s3_main_script_path(self) -> str:
        return f"{self.s3_path}/main.py"

    @property
    def s3_py_paths(self) -> List[str]:
        py_s3_paths = []
        for py_file in self.py_files:
            relative_path = py_file.relative_to(self.job_path)
            py_s3_path = f"{self.s3_path}/{relative_path}"
            py_s3_paths.append(py_s3_path)
        return py_s3_paths

    @property
    def s3_jar_paths(self) -> List[str]:
        jar_s3_paths = []
        for jar_file in self.jar_files:
            relative_path = jar_file.relative_to(self.job_path)
            jar_s3_path = f"{self.s3_path}/{relative_path}"
            jar_s3_paths.append(jar_s3_path)
        return jar_s3_paths

    @classmethod
    def new(cls, job_name: str) -> SuperglueJobType:
        return cls(job_name)

    @classmethod
    def get(cls, job_name: str) -> SuperglueJobType:
        job = cls(job_name)
        if not job.job_path.exists():
            raise FileNotFoundError(f"Glue job with name {job_name} not found.")
        return job

    def render(self) -> None:
        extra_file_args = self.get_extra_file_args()
        modules = self.modules()
        config = deepcopy(self.config)

        s3_module_paths = ",".join([module.s3_zipfile_path for module in modules])

        if s3_module_paths:
            try:
                xpfa = extra_file_args["--extra-py-files"]
                extra_file_args["--extra-py-files"] = f"{xpfa},{s3_module_paths}"
            except KeyError:
                extra_file_args["--extra-py-files"] = s3_module_paths

        config["job_config"]["DefaultArguments"].update(**extra_file_args)
        config["job_config"]["Command"]["ScriptLocation"] = self.s3_main_script_path

        if self.overrides:
            for override in self.overrides:
                base_config = deepcopy(config["job_config"])
                overriden_config = self.apply_overrides(base_config, override)
                self.deployment_config["job_configs"].append(overriden_config)
        else:
            self.deployment_config["job_configs"].append(self.config.copy()["job_config"])

    def apply_overrides(self, base_config: Dict[str, Any], overrides: Dict[str, Any]) -> Dict[str, Any]:

        for key in overrides.keys():
            if isinstance(overrides[key], Dict):
                base_config[key] = self.apply_overrides(base_config[key], overrides[key])
            else:
                base_config[key] = overrides[key]
        return base_config

    def create_or_update(self) -> None:
        glue_client = boto3.client("glue")

        for config in self.deployment_config["job_configs"]:
            try:
                # check if the glue job exists
                job_exists = glue_client.get_job(JobName=config["Name"])

            except botocore.exceptions.ClientError:
                # the job does not exist, set to None to create it.
                print(f"the job {config['Name']} does not exist. It will be created")
                job_exists = None

            if job_exists:  # then update the job definition
                print(f"the job {config['Name']} exists. It will be updated")
                # create and update glue api have different parameters for job name, so pop the name param
                # out of our config and pass it to the 'JobName' parameter of the update api.
                params = config.copy()
                job_name = params.pop("Name")

                # request a job update, if it fails a client exception is raised.
                _ = glue_client.update_job(JobName=job_name, JobUpdate=params)

            else:
                # otherwise create the job for the first time
                # if it fails a client exception is raised.
                _ = glue_client.create_job(**config)

    def modules(self) -> SuperglueComponentList:
        component_list = SuperglueComponentList()
        for name, meta in self.superglue_modules.items():
            module = SuperglueModule.from_version(module_name=name, version_number=int(meta["version_number"]))
            component_list.append(module)
        return component_list

    def get_extra_file_args(self) -> Dict[str, str]:
        extra_file_args = {}

        if self.s3_py_paths:
            s3_py_path_str = ",".join(self.s3_py_paths)
            extra_file_args["--extra-py-files"] = s3_py_path_str

        if self.s3_jar_paths:
            s3_jar_path_str = ",".join(self.s3_jar_paths)
            extra_file_args["--extra-jars"] = s3_jar_path_str

        return extra_file_args

    def deploy(self) -> None:
        self.append_version()
        self.generate_deployment_yml()
        self.sync()
        self.create_or_update()

    def delete(self) -> None:
        pass

    def save(self, **kwargs) -> None:

        if not self.job_path.exists():

            self.job_path.mkdir(exist_ok=True)
            self.pys_path.mkdir(exist_ok=True)
            self.jars_path.mkdir(exist_ok=True)

            jinja = self.get_jinja_environment()

            # set config content
            config_template = jinja.get_template("job_config.template.yml")
            config_content = config_template.render(
                iam_role=self.iam_role, job_name=self.name, s3_main_script_path=self.s3_main_script_path
            )

            self.config_file.touch(exist_ok=True)
            self.config_file.write_text(config_content)

            # set main script content
            main_script_template = jinja.get_template("main.template.py")
            main_script_content = main_script_template.render()

            self.main_script_file.touch(exist_ok=True)
            self.main_script_file.write_text(main_script_content)

            self.version_file.touch(exist_ok=True)
            self.overrides_file.touch(exist_ok=True)

            self.save_version_file()
            self.save_tests()

    def save_deployment_config(self) -> None:
        self.deployment_config_file.touch(exist_ok=True)
        yaml.dump(self.deployment_config, self.deployment_config_file.open(mode="w"), Dumper=NoAnchorsDumper)

    def generate_deployment_yml(self) -> None:
        if self.deployment_config_file.exists():
            self.deployment_config_file.unlink()
        self.render()
        self.save_deployment_config()

    def save_tests(self) -> None:
        jinja = self.get_jinja_environment()
        test_template = jinja.get_template("job_test.template.py.txt")
        test_content = test_template.render(job=self.name)

        self.job_test_path.mkdir(parents=True, exist_ok=True)
        self.job_tests_file.touch(exist_ok=True)
        self.job_tests_file.write_text(test_content)



import boto3
import inspect
from types import ModuleType
from typing import List, Any, Callable, Type
from superglue.cli.base import BaseSuperglueCommand
from superglue.environment.variables import SUPERGLUE_AWS_ACCOUNT


def get_commands(module: ModuleType) -> List[Type[BaseSuperglueCommand]]:
    commands = []
    for _, _class in inspect.getmembers(module, inspect.isclass):
        if _class != BaseSuperglueCommand:
            if issubclass(_class, BaseSuperglueCommand):
                commands.append(_class)
    return commands


def validate_account(func: Callable) -> Callable:
    def wrapper(*args, **kwargs) -> Any:
        sts = boto3.client("sts")
        account_id = int(sts.get_caller_identity()["Account"])

        if account_id != SUPERGLUE_AWS_ACCOUNT:
            print(f"superglue expects account {SUPERGLUE_AWS_ACCOUNT} but account is {account_id}")
            exit(1)
        print(f"\n -- Using AWS Account {SUPERGLUE_AWS_ACCOUNT} --")

        return func(*args, **kwargs)

    return wrapper



from pathlib import Path
from typing import List, TypeVar
from superglue.environment.config import TESTS_PATH
from superglue.core.components.base import BaseSuperglueComponent


SuperglueTestsType = TypeVar("SuperglueTestsType", bound="SuperglueTests")


class SuperglueTests(BaseSuperglueComponent):
    def __init__(self) -> None:
        super(SuperglueTests, self).__init__(
            root_dir=TESTS_PATH, component_type="superglue_tests", component_name="tests"
        )

    @property
    def tests_path(self) -> Path:
        return TESTS_PATH

    @property
    def jobs_test_dir(self) -> Path:
        return self.tests_path / "jobs"

    @property
    def modules_test_dir(self) -> Path:
        return self.tests_path / "modules"

    @property
    def conftest_file(self) -> Path:
        return self.tests_path / "conftest.py"

    @property
    def test_dirs(self) -> List[Path]:
        return [self.tests_path, self.jobs_test_dir, self.modules_test_dir]

    @classmethod
    def new(cls) -> SuperglueTestsType:
        return cls()

    def save(self) -> None:
        for test_dir in self.test_dirs:
            test_dir.mkdir(exist_ok=True)

        jinja = self.get_jinja_environment()

        conftest_template = jinja.get_template("conftest.template.py")

        self.conftest_file.touch(exist_ok=True)
        self.conftest_file.write_text(conftest_template.render())



from abc import ABC, abstractmethod
from typing import TypeVar
from argparse import Namespace
from superglue.core.components.project import SuperglueProject

SuperglueCommandType = TypeVar("SuperglueCommandType", bound="BaseSuperglueCommand")


class BaseSuperglueCommand(ABC):

    args = {}
    help = ""

    def __init__(self, cli_args: Namespace) -> None:
        self.cli_args = cli_args
        self.project = SuperglueProject()

    @classmethod
    def method(cls) -> str:
        return cls.__name__.lower()

    @abstractmethod
    def __call__(self) -> None:
        pass



from superglue.cli import parse_args
from dotenv import load_dotenv


def main() -> None:

    load_dotenv()
    cli_args = parse_args()
    command = cli_args.command(cli_args=cli_args)
    command()


if __name__ == "__main__":
    print("foo")
    """
    This is here to allow execution by referencing the project directory instead of a .py file.
    """
    main()



import os
import sys
from pathlib import Path
from dotenv import load_dotenv


# TODO: There has to be something better than this hack
program_name = Path(sys.argv[0]).parent.parts[-1]

if program_name == "pytest":
    load_dotenv(".test.env", override=True)
else:
    # load the environment file
    env_path = Path.cwd() / ".env"
    load_dotenv(env_path.as_posix())


SUPERGLUE_S3_BUCKET = os.getenv("SUPERGLUE_S3_BUCKET")
SUPERGLUE_IAM_ROLE = os.getenv("SUPERGLUE_IAM_ROLE")

# For simple account validation
try:
    SUPERGLUE_AWS_ACCOUNT = int(os.getenv("SUPERGLUE_AWS_ACCOUNT"))
except TypeError:
    SUPERGLUE_AWS_ACCOUNT = None

# keep this as we may want logging
SUPERGLUE_LOGGER_DIR = Path(os.getenv("SUPERGLUE_LOGGER_FILE", "./logs"))


def validate_environment() -> None:

    if not SUPERGLUE_S3_BUCKET:
        raise EnvironmentError("SUPERGLUE_S3_BUCKET not set in environment")

    if not SUPERGLUE_IAM_ROLE:
        raise EnvironmentError("SUPERGLUE_IAM_ROLE not set in environment")



from typing import List
from superglue.core.components.base import SuperglueComponentType


class SuperglueComponentList(list):
    def deployable(self) -> List[SuperglueComponentType]:
        return [c for c in self if c.is_deployable]

    def locked(self) -> List[SuperglueComponentType]:
        return [c for c in self if c.is_locked]

    def unlocked(self) -> List[SuperglueComponentType]:
        return [c for c in self if c.is_unlocked]

    def are_locked(self) -> bool:
        locked_status = [c.is_locked for c in self]
        return False not in locked_status

    def are_unlocked(self) -> bool:
        return not self.are_locked()

    def are_packaged(self) -> bool:
        packaged_status = [c.is_packaged for c in self]
        return False not in packaged_status

    def are_not_packaged(self) -> bool:
        return not self.are_packaged()

    def versions_match(self) -> bool:
        version_status = [c.version_in_sync for c in self]
        return False not in version_status



from superglue.cli.utils import validate_account
from superglue.cli.base import BaseSuperglueCommand
from superglue.cli.messages import Messages

__version__ = "0.14.0"


class Version(BaseSuperglueCommand):

    help = "--> Print the current version of superglue and exit."

    def __call__(self) -> None:
        print(f"The AWS Glue Deployment Utility -- superglue version :: {__version__}")


class Account(BaseSuperglueCommand):

    help = "--> Print the AWS account number that superglue is configured to use."

    @validate_account
    def __call__(self) -> None:
        pass


class Init(BaseSuperglueCommand):

    help = "--> Creates a new superglue project structure."

    def __call__(self) -> None:
        self.project.create()
        Messages.project_initialized()


class New(BaseSuperglueCommand):

    help = "--> Creates a new superglue job or module"

    args = {
        ("component_type",): {"choices": ["job", "module"], "help": "Can be either job or module"},
        ("-n", "--name"): {"required": True, "help": "The name of the new component to create"},
    }

    def __call__(self) -> None:

        component = getattr(self.project, self.cli_args.component_type).new(self.cli_args.name)
        messages = Messages(component)

        if component.component_path.exists():
            messages.component_exists()
            exit(1)

        else:
            component.save()
            messages.component_created()
            exit(0)


class Check(BaseSuperglueCommand):

    help = "--> Used to check the status of the superglue project before a deployment"

    @validate_account
    def __call__(self) -> None:

        if not self.project.is_locked():
            Messages.no_deployment()
            exit(1)

        if self.project.modules.are_not_packaged():
            Messages.not_packaged()
            exit(1)

        if not self.project.versions_match():
            Messages.version_missmatch()
            exit(1)

        else:
            Messages.yes_deployment()


class Lock(BaseSuperglueCommand):

    help = "--> Lock superglue jobs and modules to the next version if they have active edits."

    def __call__(self) -> None:

        Messages.locking_jobs()

        if self.project.jobs.are_unlocked():
            for job in self.project.jobs.unlocked():
                Messages.locking_job(job)
                job.lock()
        else:
            Messages.all_jobs_locked()

        Messages.locking_modules()

        if self.project.modules.are_unlocked():
            for module in self.project.modules.unlocked():
                Messages.locking_module(module)
                module.lock()
        else:
            Messages.all_modules_locked()


class Package(BaseSuperglueCommand):

    help = "--> Packages all superglue jobs and modules which have been edited since the last package was issued."

    args = {("-p", "--purge"): {"action": "store_true", "default": False}}

    def __call__(self) -> None:

        if self.cli_args.purge:
            self.purge()

        else:
            self.lock_check()
            self.package()

    def purge(self) -> None:
        for module in self.project.modules:
            module.remove_zipfile()
            Messages.removed_zipfile(module)
        Messages.purge_complete()
        exit(0)

    def lock_check(self) -> None:
        if self.project.modules.are_unlocked():
            Messages.modules_not_locked()
            exit(1)

    def package(self) -> None:
        for module in self.project.modules:
            module.package()
            Messages.packaging_module(module.name)
        Messages.packaging_complete()
        exit(0)


class Status(BaseSuperglueCommand):

    help = "--> Prints the current status of all superglue jobs and modules."

    args = {
        ("-m", "--modules"): {"action": "store_true", "default": False, "dest": "only_modules"},
        ("-j", "--jobs"): {"action": "store_true", "default": False, "dest": "only_jobs"},
    }

    @validate_account
    def __call__(self) -> None:
        table = self.project.get_pretty_table()

        if not self.cli_args.only_jobs:
            for module in self.project.modules:
                table.add_row(module.pretty_table_row)

        if not self.cli_args.only_modules:
            for job in self.project.jobs:
                table.add_row(job.pretty_table_row)

        print(table)


class Deploy(BaseSuperglueCommand):

    help = "--> Deploys all superglue jobs and modules which have been changed since the last issued deployment."

    args = {
        ("-d", "--dry"): {
            "action": "store_true",
            "default": False,
            "help": "Print a dry run to the terminal of what superglue would deploy",
        }
    }

    @validate_account
    def __call__(self) -> None:

        if not self.project.is_locked():
            Messages.no_deployment()
            exit(1)

        if self.project.modules.are_not_packaged():
            Messages.not_packaged()
            exit(1)

        if self.cli_args.dry:
            Messages.dry_run()
            self.dry_module_deploy()
            self.dry_job_deploy()
            exit(0)

        Messages.yes_deployment()
        self.module_deploy()
        self.job_deploy()

    def dry_module_deploy(self) -> None:
        for module in self.project.modules.deployable():
            Messages.module_deploy(module, dry=True)

    def dry_job_deploy(self) -> None:
        for job in self.project.jobs.deployable():
            job.generate_deployment_yml()
            Messages.job_deploy(job, dry=True)

    def module_deploy(self) -> None:
        for module in self.project.modules.deployable():
            module.deploy()
            Messages.module_deploy(module)

    def job_deploy(self) -> None:
        for job in self.project.jobs.deployable():
            job.deploy()
            Messages.job_deploy(job)


class Refresh(BaseSuperglueCommand):

    help = "--> Refresh all local version numbers with the latest versions stored in S3."

    def __call__(self) -> None:

        for component in self.project.components:
            remote_version = component.fetch_s3_version_number()
            if int(remote_version) != int(component.version_number):
                Messages.updating_local_version(component, remote_version)
                component.save_version_file(version_number=remote_version)
            else:
                Messages.versions_in_sync(component, remote_version)


class Generate(BaseSuperglueCommand):

    help = "--> Used to generate superglue tests, and utility files"

    args = {
        ("-t", "--tests"): {"action": "store_true", "default": False},
        ("-m", "--makefile"): {"action": "store_true", "default": False},
    }

    def __call__(self) -> None:

        if self.cli_args.tests:
            for job in self.project.jobs:
                job.save_tests()

            for module in self.project.modules:
                module.save_tests()

        if self.cli_args.makefile:
            makefile = self.project.makefile.new()
            makefile.save()



from argparse import ArgumentParser
from superglue.cli import commands
from superglue.cli.utils import get_commands


def parse_args():

    # This is the root node of the main parser tree.
    parser = ArgumentParser()
    sub_parser = parser.add_subparsers(dest="command")
    sub_parser.required = True

    # set parser for all root level actions in the format
    # superglue "action" <options>
    for command in get_commands(commands):
        p = sub_parser.add_parser(command.method(), help=command.help)
        for args, kwargs in command.args.items():
            p.add_argument(*args, **kwargs)
        p.set_defaults(command=command)
    return parser.parse_args()



from argparse import ArgumentParser, Action, Namespace
from typing import Any
from superglue.core.components.project import SuperglueProject

#
#
# def validate_input_string(input_string: str, message: str) -> None:
#     forbidden = "!" "#$%&'()*+,./:;<=>?@[\\]^`{|}~"
#     for char in input_string:
#         if char in forbidden:
#             print(message)
#             exit()
#
#


def exit_if_job_exists(job_name: str) -> None:

    project = SuperglueProject()
    job = project.job.new(job_name)

    if job.job_path.exists():
        print(f"The job {job.job_name} already exists")
        exit(1)


#
#
# def check_module_exists(module_name: str) -> None:
#
#     project = SuperGlueProject()
#     module = SuperGlueModule(project.shared_path, module_name)
#
#     if not module.module_path.exists():
#         print(f"The module {module.module_name} does not exist")
#         exit()
#
#
# class ValidateJobCommandName(Action):
#     def __call__(self, parser: ArgumentParser, namespace: Namespace, values: Any, option_string: str = None):
#         validate_input_string(values, f"The {values} is not allowed in job names")
#
#         # The job of course cannot already exist if we ar calling the new cmd
#         if not namespace.func.__name__ == "new":
#             check_job_exists(values)
#
#         setattr(namespace, self.dest, values)
#
#
# class ValidateModuleCommandName(Action):
#     def __call__(self, parser: ArgumentParser, namespace: Namespace, values: Any, option_string: str = None):
#
#         validate_input_string(values, f"The {values} is not allowed in job names")
#
#         # The module of course cannot already exist if we ar calling the new cmd
#         if not namespace.func.__name__ == "new":
#             check_module_exists(values)
#
#         setattr(namespace, self.dest, values)
